#!/usr/bin/perl

##################################################
# STUFF WE MAY OR MAY NOT USE
##################################################
use strict;
#use warnings;
use Getopt::Std;
use Nmap::Parser::XML;
use Parallel::ForkManager;
use LWP::Simple;

##################################################
# This copyright applies to all code 
# included in this distribution.
#
# Copyright (C) 2002-2003 John Burkhart / Klinge-C01
#
# This program is free software; 
# you can redistribute it and/or modify it under
# the terms of the GNU General Public License
# as published by the Free Software Foundation; 
# either version 2  of the License,
# or (at your option) any later version.
#
# This program is distributed in the hope that it 
# will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General 
# Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - 
# Suite 330, Boston, MA  02111-1307, USA.
#
# Contact Information:
#  John Burkhart (jtburkh@sdf.lonestar.org)
#
##################################################


##################################################
# Requires:
# - Nmap::Parser::XML - 
#   	wget http://search.cpan.org/dist/Nmap-Parser-XML/Nmap-Parser-XML-0.66_1.tar.gz
# 	Nmap-Parser-XML-0.66_1.tar.gz was a developers release that had tcp_ports and extras that I wanted.
# - Parallel::ForkManager -
# - LWP::Simple -
#
# - nmap 3.0+
# - ldapsearch (along with a good ldap.conf) 
# - /bin/date
# - perl
# 
# 
#
# ToDo:
# X.  Finalize a fork() function. 10/09/03
# X.  Output data to a file. 10/07/03
# 3.  Use LDAP pm's.
# 4.  Use Perl's std for grepping/cutting/sorting/removing data in LDAP data.
# X.  Add in nmap scanning compliment to ldapsearch. 10/06/03
#
##################################################

##################################################
# CONFIGURATION OPTIONS YOU PROBABLY WANT TO SET
##################################################
my $STORAGE = '/tmp'; 				# This is the Directory Where Your Scans Will Sit
my $LDAP_KEY_FOR_IP_ADDRESSES = 'iphostnumber'; # This is where IP Addresses are stored in my ldap directory.
my $LDAP_KEY_FOR_MACADDRESS = 'macaddress'; 	# This is where MACs are stored in my ldap directory.
my $FORKS_ALLOWED = 10;


##################################################
# MY VERSIONING SYSTEM
##################################################
my $VERSION = "1.0";


##################################################
# DECLARE OPTIONS
##################################################
my %opts;
getopts("t:p:l:n:o:g:hv", \%opts);


##################################################
# STATIC FILE NAME FOR HOLDING NMAP XML DATA.
# NEEDS AUTO-ALPHA UPPER
##################################################
my $file_date = `/bin/date "+%m%d%y.xml"`;


##################################################
# DECLARE THE TOOL THAT WE WANT TO USE.
##################################################
if (! $opts{t}) { usage() }
my $TOOL = $opts{t};


##################################################
# DECLARE OUR LDAP OR nmap READABLE IP ADDRESS 
# OR RANGE.
# Note: make sure your ldap.conf works first. 
# See 00-README-00
##################################################
if (!$opts{l} && !$opts{n}) 
{ 
	print "Use either the 'n' or 'l' option\n"; 
	usage(); 
}

my $IPHOSTNUMBER = $opts{l};
my $nmapIP = $opts{n};


##################################################
# DECLARE THE PORT WE ARE GOING AFTER.
##################################################
if (! $opts{p}) { usage() }
my $PORT = $opts{p};
print "\n-> -> -> -> ->  PORT: $PORT \n";


##################################################
# DECLARE AN OPTIONAL COMPONENT TO THE TOOL THAT 
# WE CHOSE TO USE (0).
##################################################
my $FLAG = $opts{g};
print "\n-> -> -> -> ->  OPTION: $FLAG \n";


##################################################
# DECLARE AN OPTIONAL OUTPUT FILE.
##################################################
my $OUTLOC = $opts{o};
print "\n-> -> -> -> ->  OUTPUT FILE: $OUTLOC \n";


##################################################
# HELP
##################################################
if ($opts{h}) { usage(); show_info(); }


##################################################
# SHOW INFORMATION 
##################################################
if ($opts{v}) { show_info() }


##################################################
# USAGE 
##################################################
sub usage {
        print STDERR "\n";
	print STDERR "+---<< USAGE >>---------------------------------------------== VICTIMIZER ==--+\n\n";
	print STDERR "		-t <Full Path To Executable>						\n";
	print STDERR "		-g <SPECIFIC TOOL OPTION 0>						\n";
	print STDERR "		-l <IP Range that ldapsearch Understands (*)>				\n";
	print STDERR "		-n <IP Range that nmap Understands>					\n";
	print STDERR "		-p <Port Number or Range that nmap Understands>				\n";
	print STDERR "		-o <Results Directory - Output>						\n";
	print STDERR "		-h <This Usage Information>						\n";
	print STDERR "											\n";
	print STDERR "Example: $0 -t /usr/bin/rpcinfo -l '192.168.*' -p 111 -g '-p' 			\n";
	print STDERR "Example: $0 -t showmount -n 192.168.0-255.1-254 -p 2049 -g '-e' 			\n";
	print STDERR "Example: $0 -t /your/exploit -l '192.168.*' -p 139 -g '-h' 			\n";
	print STDERR "Example: $0 -t nbaudit -n '192.168.*' -p 139 					\n";
	print STDERR "Example: $0 -t nbaudit -l '192.168.1.10*' -p 139 -o /home/surefoot 		\n";
	print STDERR "\n+---<< VICTIMIZER >>---------------------------------------------== USAGE ==--+\n\n";
	print STDERR "\n\n\n";

	exit(0);
}


##################################################
# SHOW INFORMATION
##################################################
sub show_info {
	print STDERR "\n";
	print "+---<< INFO >>----------------------------------------------== VICTIMIZER ==--+\n\n";
	print " Name:           victimizer.pl							\n";
	print " Author:         John Burkhart <jtburkh@sdf.lonestar.org>			\n";
	print " Version:        $VERSION							\n";
	print " Tested On: 	OpenBSD 3.3 							\n\n";
	print "+---<< VICTIMIZER >>----------------------------------------------== INFO ==--+\n\n";
	print " victimizer.pl is made to take advantage of tools that I use everyday that	\n";
        print " don't have options for hooking into more than one IP address at a time.		\n"; 
        print " For instance I may need to rpcinfo a bunch of machines, but rpcinfo doesn't	\n";
        print " allow me to do this. victimizer.pl does by giving me the option to call any	\n"; 
        print " network program I want and feed it ip addresses that I want. Multithreading	\n"; 
	print " is built in to help increase the speed of the scan in regards to system calls   \n";
        print " Initially you have the option to choose your targets from ldap or through	\n";
	print " a standard nmap scan.  This utility is still very basic and will be re-coded.	\n";
	print " I'm not a perl programmer.............so this may not work.			\n";
        print "											\n";
	print "+---<< VICTIMIZER >>----------------------------------------------== INFO ==--+\n\n";
}


##################################################
# SECTION 1/2A - GATHERING LDAP IPS AND SCANNING.
# FU-UGLY - TO BE UPDATED VIA PM.
##################################################
if ($opts{l}) 
{
	print "\n-> -> -> -> ->  CALL THE LDAP SLOTH...\n";
	system "/usr/local/bin/ldapsearch -LLL '(&($LDAP_KEY_FOR_IP_ADDRESSES=$IPHOSTNUMBER)($LDAP_KEY_FOR_MACADDRESS=*))' iphostnumber > /tmp/ldap";
	system "grep $LDAP_KEY_FOR_IP_ADDRESSES /tmp/ldap > /tmp/ldap1";
	system "cut -f 2 -d : /tmp/ldap1 > /tmp/ldap2";
	system "sort /tmp/ldap2 > ./Hosts";
	system "rm /tmp/ldap /tmp/ldap1 /tmp/ldap2";
	print "\n-> -> -> -> ->  VICTIMS READY...\n";
	print "\n-> -> -> -> ->  nmap WARMING UP...\n";
	system "nmap --max_parallelism 100 -n -sS -iL Hosts -T Aggressive -p $PORT -oX `date '+%m%d%y.xml'`";
}


##################################################
# SECTION 1/2B - GATHERING IPS AND SCANNING.
# FU-UGLY - TO BE UPDATED VIA PM.
##################################################
if ($opts{n}) 
{
	print "\n-> -> -> -> ->  USING nmap TO HUNT VICTIMS...\n\n";
	system "nmap --max_parallelism 100 -n -sS -p $PORT -T Aggressive -oX `date '+%m%d%y.xml'` $nmapIP";
}


##################################################
# SECTION 3 - MAKING A LOGFILE TO SHOW WORK DONE.
##################################################
my $date;
my $logfile = `/bin/date "+%m%d%y.log"`;
chomp($date = `date`);
open(TMPLOG, ">>./$logfile") or die "CANNOT WRITE TO LOGFILE: $!";
print TMPLOG $date, " started\n";
close TMPLOG;


##################################################
# SETTING UP STORAGE FOR OUR OUTPUT 
##################################################
my $day;
chomp($day = `date +%m%d%y`);
my $time;
chomp($time = `date +%m%d%y.%H%M`);

my $STO;
my $DAILY_STORAGE;

if ($opts{o}) 
{
	my $STO = "$OUTLOC";
	mkdir($STO, 0777) unless (-d "$STO");
	print "\n-> -> -> -> ->  ALL OUTPUT IS STORED IN $STO \n\n";
}
else
{
	$DAILY_STORAGE = "$STORAGE/$day";
	mkdir($DAILY_STORAGE, 0777) unless (-d "$DAILY_STORAGE");
	print "\n-> -> -> -> ->  ALL OUTPUT IS STORED IN $DAILY_STORAGE \n";
}

##################################################
# READY A PARSER FOR XML/NMAP
##################################################
my $parse = new Nmap::Parser::XML;
$parse->parsefile($file_date);


##################################################
# MAIN PROCESSING
##################################################

my $executioner=new Parallel::ForkManager($FORKS_ALLOWED);

for my $host_obj ($parse->get_host_objects())
{
	my @victim = $host_obj->addr();
	my $state = $host_obj->tcp_ports($PORT);

	for my $victim (@victim) 
	{
		$executioner->start and next;
		#my ($fn)= $victim =~ /^.*\/(.*?)$/;
		if ($state eq 'open')
		{
			if ($opts{o})
			{
				open (STDOUT, ">$OUTLOC/$victim.$time.out") or die "Can't open $OUTLOC/$victim.$time.out: $!";
				print "$victim SCANNED ON $time WITH $TOOL \n";
				print "-------------------------------------------------------------------\n";
				close STDOUT;
				print "Executing $TOOL $FLAG $victim $OUTLOC \n";
				exec "$TOOL $FLAG $victim >> $OUTLOC/$victim.$time.out \n";
			}
			else
			{

				open (STDOUT, ">$DAILY_STORAGE/$victim.$time.out") or die "Can't open $DAILY_STORAGE/$victim.$time.out: $!";
				print "$victim SCANNED ON $time WITH $TOOL \n";
				print "-------------------------------------------------------------------\n";
				close STDOUT;
				exec "$TOOL $FLAG $victim >> $DAILY_STORAGE/$victim.$time.out \n";
			}
		}
		else
		{
			#print "NOTHING TO DO FOR $victim BECAUSE $PORT STATE IS $state \n";
		}
		$executioner->finish;
	}
};



##################################################
# END LOG
##################################################
chomp($date = `date`);
open(TMPLOG, ">>./$logfile") or warn "$!";
print TMPLOG $date, " stopped\n";
close TMPLOG;
exit;

# 47 72 65 65  74 73 20 67  6F 20 6F 75  74 20 74 6F
# 20 53 68 33  72 69 2C 20  37 69 37 37  69 34 6E 2C
# 20 6C 69 74  74 6C 65 2D  62 69 74 7A  2C 20 61 6C
# 70 68 61 66  69 67 68 74  2C 20 6C 74  6C 77 30 6C
# 66 20 26 20  69 6E 73 61  6E 65 2E 0A  0A

# EOF
